import { Plugin } from 'ckeditor5/src/core';
import { TooltipView, Template } from 'ckeditor5/src/ui';
import UpdateMediaRevisionCommand from './command'
import MediaRevisionsRepository from './repository'

export default class MediaRevisionsEditing extends Plugin {

  static get requires() {
    return ['DrupalMediaEditing', MediaRevisionsRepository];
  }

  /**
   * @inheritdoc
   */
  static get pluginName() {
    return 'MediaRevisionsEditing';
  }

  init() {
    const {editor} = this;
    editor.model.schema.extend('drupalMedia', {
      allowAttributes: ['entityRevision', 'entityIsLatestRevision'],
    });
    const mediaEditing = editor.plugins.get('DrupalMediaEditing');
    mediaEditing.attrs['entityRevision'] = 'data-entity-revision';

    const { conversion } = editor;
    const attributeMapping = {
      model: {
        key: 'entityRevision',
        name: 'drupalMedia',
      },
      view: {
        name: 'drupal-media',
        key: 'data-entity-revision',
      },
    };
    conversion.for('dataDowncast').attributeToAttribute(attributeMapping);
    conversion.for('upcast')
      .attributeToAttribute(attributeMapping)
      .add((dispatcher) => {
        dispatcher.on(
          'element:drupal-media',
          (event, data) => {
            const [modelElement] = data.modelRange.getItems();
            const metadataRepository = this.editor.plugins.get(
              'MediaRevisionsRepository',
            );
            metadataRepository
              .getRevisionMetadata(modelElement)
              .then(metadata => {
                if (!modelElement) {
                  return;
                }
                editor.model.enqueueChange(
                  { isUndoable: false },
                  (writer) => {
                    writer.setAttribute(
                      'entityIsLatestRevision',
                      metadata.isLatest,
                      modelElement,
                    );
                  },
                );
              })
          },
          { priority: 'lowest' },
        )
      })
    conversion.for('editingDowncast').add((dispatcher) => {
      dispatcher.on(
        'attribute:entityRevision',
        (event, data) => {
          if (data.attributeOldValue === null || data.attributeOldValue === data.attributeNewValue) {
            return;
          }
          const metadataRepository = this.editor.plugins.get(
            'MediaRevisionsRepository',
          );
          metadataRepository
            .refreshModelMetadata(data.item)
            .then(metadata => {
              if (!data.item) {
                return;
              }
              editor.model.enqueueChange(
                { isUndoable: false },
                (writer) => {
                  writer.setAttribute(
                    'entityIsLatestRevision',
                    metadata.isLatest,
                    data.item,
                  );
                },
              );
            })
        },
      )
      dispatcher.on(
        'attribute:entityIsLatestRevision',
        (event, data, conversionApi) => {
          const { writer, mapper } = conversionApi;
          const container = mapper.toViewElement(data.item);

          if (data.attributeNewValue === true) {
            const existingError = Array.from(container.getChildren()).find(
              (child) => child.getCustomProperty('entityRevisionWarning'),
            );
            if (existingError) {
              writer.remove(existingError)
            }
            return;
          }

          const message = Drupal.t(
            'This media item has a newer version available.',
          );

          const tooltip = new TooltipView();
          tooltip.text = message;
          tooltip.position = 'sw';

          const html = new Template({
            tag: 'span',
            children: [
              {
                tag: 'span',
                attributes: {
                  class: 'drupal-media__mediarevision-update-icon',
                },
              },
              tooltip,
            ],
          }).render();
          const error = writer.createRawElement(
            'div',
            {
              class: 'drupal-media__mediarevision-update',
            },
            (domElement, domConverter) => {
              domConverter.setContentOf(domElement, html.outerHTML);
            },
          );
          writer.setCustomProperty('entityRevisionWarning', true, error);

          writer.insert(writer.createPositionAt(container, 0), error);
        },
        { priority: 'low' },
      );
    });

    editor.commands.add(
      'updateMediaRevision',
      new UpdateMediaRevisionCommand(editor),
    );
  }

}
