<?php

namespace Drupal\acquia_dam\Controller;

use Drupal\Core\Access\CsrfTokenGenerator;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\media\MediaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller for the media revision CKEditor plugin.
 */
final class EditorMediaRevisionController implements ContainerInjectionInterface {

  /**
   * The CSRF token service.
   *
   * @var \Drupal\Core\Access\CsrfTokenGenerator
   */
  private $csrfToken;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  private $entityRepository;

  /**
   * Constructs a new EditorMediaRevisionController object.
   *
   * @param \Drupal\Core\Access\CsrfTokenGenerator $csrfToken
   *   The CSRF token service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity repository.
   */
  public function __construct(CsrfTokenGenerator $csrfToken, EntityTypeManagerInterface $entityTypeManager, EntityRepositoryInterface $entityRepository) {
    $this->csrfToken = $csrfToken;
    $this->entityTypeManager = $entityTypeManager;
    $this->entityRepository = $entityRepository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('csrf_token'),
      $container->get('entity_type.manager'),
      $container->get('entity.repository'),
    );
  }

  /**
   * Returns JSON response to determine if a media embed is the latest revision.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function isLatestRevision(Request $request): JsonResponse {
    $header = 'X-Drupal-MediaRevision-CSRF-Token';
    if (!$request->headers->has($header)) {
      throw new AccessDeniedHttpException('X-Drupal-MediaRevision-CSRF-Token header is missing.');
    }
    $token = $request->headers->get($header);
    if (!$this->csrfToken->validate($token, $header)) {
      throw new AccessDeniedHttpException('X-Drupal-MediaRevision-CSRF-Token is invalid.');
    }

    $uuid = $request->query->get('uuid', '');
    $revision_id = $request->query->get('revisionId', '');
    if ($uuid === '' || $revision_id === '') {
      throw new NotFoundHttpException();
    }

    $media = $this->entityRepository->loadEntityByUuid('media', $uuid);
    if (!$media instanceof MediaInterface) {
      throw new NotFoundHttpException();
    }
    $revision = $this->entityTypeManager->getStorage('media')->loadRevision($revision_id);
    if (!$revision instanceof MediaInterface) {
      throw new NotFoundHttpException();
    }

    if ($revision->uuid() !== $media->uuid()) {
      throw new NotFoundHttpException();
    }

    return JsonResponse::create([
      'isLatest' => $revision->isLatestRevision(),
    ])
      ->setPrivate()
      // Allow the end user to cache this response for 5 minutes.
      ->setMaxAge(300);
  }

}
