<?php

namespace Drupal\acquia_dam;

use Drupal\acquia_dam\Entity\MediaSourceField;
use Drupal\acquia_dam\Plugin\Field\FieldType\AssetItem;
use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Url;
use Drupal\media\MediaInterface;

/**
 * Create system specific embed codes.
 */
final class EmbedCodeFactory implements TrustedCallbackInterface {

  /**
   * Returns array containing select options for select render.
   *
   * @param string|null $asset_type
   *   Asset type.
   *
   * @return string[]
   *   Select options for given type or full mapping if asset type not provided.
   */
  public static function getSelectOptions(string $asset_type = NULL): array {
    // Generated url won't have the download option.
    // https://www.drupal.org/project/drupal/issues/2984272 Core issue where
    // "." in the query parameter gets replaced with "_" so the url with
    // "t.download=true" will be replaced with "t_download=true".
    $image_styles = [];
    /** @var \Drupal\acquia_dam\ImageStyleHelper $image_style_helper */
    $image_style_helper = \Drupal::service('acquia_dam.image_style_support');
    foreach ($image_style_helper->getAllowedImageStyles() as $styles) {
      $image_styles[$styles->id()] = $styles->label();
    }
    $image_styles = ['original' => 'Original'] + $image_styles;
    $mapping = [
      'pdf' => [
        'inline_view_download' => 'Inline viewer with download',
        'inline_view' => 'Inline viewer without download',
        'link_text_download' => 'Text linked to viewer with download',
        'link_text' => 'Text linked to viewer without download',
        'link_thumbnail_download' => 'Thumbnail linked to viewer with download',
        'link_thumbnail' => 'Thumbnail linked to viewer without download',
      ],
      'video' => [
        'inline_view_download' => 'Inline player with download',
        'inline_view' => 'Inline player without download',
        'link_thumbnail_download' => 'Thumbnail linked to player with download',
        'link_thumbnail' => 'Thumbnail linked to player without download',
        'video_stream' => 'Video without player',
      ],
      'spinset' => [
        'inline_view' => 'Inline viewer without download',
        'link_text' => 'Text linked to viewer without download',
      ],
      'image' => $image_styles,
      'documents' => [
        'inline_view_download' => 'Inline viewer with download',
        'link_text_download' => 'Text linked to viewer with download',
        'link_thumbnail_download' => 'Thumbnail linked to viewer with download',
        'link_thumbnail' => 'Thumbnail linked to viewer without download',
      ],
    ];

    return $asset_type ? $mapping[$asset_type] : $mapping;
  }

  /**
   * Returns a render array based on the parameters.
   *
   * @param array $embeds
   *   All embed codes of the asset.
   * @param string $format
   *   Embed code id for formatting.
   * @param \Drupal\media\MediaInterface $media
   *   Media instance.
   *
   * @return array
   *   Render array.
   */
  public static function renderAsset(array $embeds, string $format, MediaInterface $media): array {
    /** @var \Drupal\acquia_dam\Plugin\media\Source\Asset $source */
    $source = $media->getSource();
    switch ($source->getDerivativeId()) {
      case 'documents':
      case 'pdf':
        $embed = self::renderDocumentsPdfAsset($embeds, $format, $media);
        break;

      case 'video':
        $embed = self::renderVideoAsset($embeds, $format, $media);
        break;

      case 'spinset':
        $embed = self::renderSpinsetAsset($embeds, $format, $media);
        break;

      case 'image':
        $asset_field = $media->get(MediaSourceField::SOURCE_FIELD_NAME)->first();
        assert($asset_field instanceof AssetItem);
        $image_properties = $source->getMetadata($media, 'image_properties');
        if ($format == 'original') {
          $embed = [
            '#theme' => 'image',
            '#uri' => "acquia-dam://$asset_field->asset_id/$asset_field->version_id",
            // @todo fetch alt from metadata for a proper value.
            '#alt' => $media->label(),
            '#width' => $image_properties['width'],
            '#height' => $image_properties['height'],
          ];
        }
        else {
          $embed = [
            '#theme' => 'image_style',
            '#uri' => "acquia-dam://$asset_field->asset_id/$asset_field->version_id.png",
            // @todo fetch alt from metadata for a proper value.
            '#alt' => $media->label(),
            '#width' => $image_properties['width'],
            '#style_name' => $format,
            '#height' => $image_properties['height'],
          ];
        }

        break;

      default:
        $embed = [];
    }

    return $embed;
  }

  /**
   * Returns a pdf/document asset render array based on the parameters.
   *
   * @param array $embeds
   *   All embed codes of the asset.
   * @param string $format
   *   Embed code id for formatting.
   * @param \Drupal\media\MediaInterface $media
   *   Media instance.
   *
   * @return array
   *   Render array.
   */
  protected static function renderDocumentsPdfAsset(array $embeds, string $format, MediaInterface $media) :array {
    switch ($format) {
      case 'original':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['original']['html']),
        ];
        break;

      case 'inline_view_download':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['document_viewer_with_download']['html']),
        ];
        break;

      case 'inline_view':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['document_viewer']['html']),
        ];
        break;

      case 'link_text_download':
        $embed = self::renderAsLink($embeds['document_viewer_with_download'], $media->getName());
        break;

      case 'link_text':
        $embed = self::renderAsLink($embeds['document_viewer'], $media->getName());
        break;

      case 'link_thumbnail_download':
        $embed = self::renderAsThumbnailLink($embeds['document_viewer_with_download'], $embeds['document_thumbnail'], $media->getName());
        break;

      case 'link_thumbnail':
        $embed = self::renderAsThumbnailLink($embeds['document_html5_viewer'], $embeds['document_thumbnail'], $media->getName());
        break;

      default:
        $embed = [];
    }

    return $embed;
  }

  /**
   * Returns a video asset render array based on the parameters.
   *
   * @param array $embeds
   *   All embed codes of the asset.
   * @param string $format
   *   Embed code id for formatting.
   * @param \Drupal\media\MediaInterface $media
   *   Media instance.
   *
   * @return array
   *   Render array.
   */
  protected static function renderVideoAsset(array $embeds, string $format, MediaInterface $media): array {
    switch ($format) {
      case 'original':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['original']['html']),
        ];
        break;

      case 'inline_view_download':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['VideoWithPlayerAndDownload']['html']),
        ];
        break;

      case 'inline_view':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['video_player']['html']),
        ];
        break;

      case 'link_thumbnail_download':
        $embed = self::renderAsThumbnailLink($embeds['VideoWithPlayerAndDownload'], $embeds['video_poster'], $media->getName());
        break;

      case 'link_thumbnail':
        $embed = self::renderAsThumbnailLink($embeds['video_player'], $embeds['video_poster'], $media->getName());
        break;

      case 'video_stream':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['video_stream']['html']),
        ];
        break;

      default:
        $embed = [];
    }

    return $embed;
  }

  /**
   * Returns a spinset asset render array based on the parameters.
   *
   * @param array $embeds
   *   All embed codes of the asset.
   * @param string $format
   *   Embed code id for formatting.
   * @param \Drupal\media\MediaInterface $media
   *   Media instance.
   *
   * @return array
   *   Render array.
   */
  protected static function renderSpinsetAsset(array $embeds, string $format, MediaInterface $media): array {
    switch ($format) {
      case 'original':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['original']['html']),
        ];
        break;

      case 'inline_view':
        $embed = [
          '#type' => 'markup',
          '#markup' => Markup::create($embeds['spinset_viewer']['html']),
        ];
        break;

      case 'link_text':
        $embed = self::renderAsLink($embeds['spinset_viewer'], $media->getName());
        break;

      default:
        $embed = [];
    }

    return $embed;
  }

  /**
   * Returns link pointing to the given asset embed.
   *
   * @param array $embed
   *   Embed code to use for the link.
   * @param string $asset_name
   *   Asset name as link name.
   *
   * @return array
   *   Render array.
   */
  protected static function renderAsLink(array $embed, string $asset_name): array {
    $link = Link::fromTextAndUrl($asset_name, Url::fromUri($embed['url'], ['attributes' => ['target' => '_blank']]));
    $render_array = $link->toRenderable();
    $render_array['#post_render'] = [[static::class, 'postRenderDamLink']];
    return $render_array;
  }

  /**
   * Returns thumbnail as a link.
   *
   * @param array $embed
   *   Embed code of the DAM asset.
   * @param array $thumbnail
   *   Thumbnail embed.
   * @param string $name
   *   File name.
   *
   * @return array
   *   Render array.
   */
  protected static function renderAsThumbnailLink(array $embed, array $thumbnail, string $name): array {
    $link_title = [
      '#theme' => 'image',
      '#width' => 300,
      '#height' => 300,
      '#uri' => $thumbnail['url'],
      '#alt' => sprintf('%s preview', $name),
    ];

    return [
      '#type' => 'container',
      '#theme_wrappers' => ['container__acquia_dam_asset'],
      'embed' => [
        '#type' => 'link',
        '#title' => $link_title,
        '#url' => Url::fromUri($embed['url'], ['attributes' => ['target' => '_blank']]),
        '#post_render' => [[static::class, 'postRenderDamLink']],
      ],
    ];
  }

  /**
   * Replace t_download within the markup with the correct key.
   *
   * @param string $markup
   *   Link markup generated for embed code.
   *
   * @return string
   *   Markup.
   */
  public static function postRenderDamLink(string $markup) {
    if (!str_contains($markup, 't_download')) {
      return $markup;
    }

    return str_replace('t_download', 't.download', $markup);
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['postRenderDamLink'];
  }

}
