<?php

namespace Drupal\acquia_dam;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\image\Entity\ImageStyle;

/**
 * Helper functions for image style operations.
 */
class ImageStyleHelper {

  /**
   * Image effect plugin ids which are currently not supported by Acquia DAM.
   */
  const UNSUPPORTED_IMAGE_EFFECT_PLUGIN_IDS = [
    'focal_point_crop',
    'focal_point_scale_and_crop',
  ];

  /**
   * Acquia DAM settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $acquiaDamSettings;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * ImageStyleHelper constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   */
  public function __construct(ConfigFactoryInterface $configFactory, EntityTypeManagerInterface $entityTypeManager) {
    $this->acquiaDamSettings = $configFactory->get('acquia_dam.settings');
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Returns all image style based on support status.
   *
   * @return array[]
   *   Supported and non-supported image styles.
   */
  public function getImageStylesBySupportStatus(): array {
    $image_styles = [
      'supported' => [],
      'not-supported' => [],
    ];

    $image_styles_storage = $this->entityTypeManager->getStorage('image_style');
    foreach ($image_styles_storage->loadMultiple() as $image_style) {
      $supported = TRUE;
      if ($effects = $image_style->getEffects()) {
        foreach ($effects as $effect) {
          if (in_array($effect->getPluginId(), self::UNSUPPORTED_IMAGE_EFFECT_PLUGIN_IDS, TRUE)) {
            $image_styles['not-supported'][] = $image_style;
            $supported = FALSE;
            break;
          }
        }
      }

      if ($supported) {
        $image_styles['supported'][] = $image_style;
      }
    }

    return $image_styles;
  }

  /**
   * Returns allowed image styles.
   *
   * @return \Drupal\image\ImageStyleInterface[]
   *   Image styles set in config or all which is supported.
   */
  public function getAllowedImageStyles(): array {
    $styles = $this->acquiaDamSettings->get('allowed_image_styles') ?? [];

    if (empty($styles)) {
      return $this->getImageStylesBySupportStatus()['supported'];
    }

    return ImageStyle::loadMultiple($styles);
  }

}
